#Moose
#mglu-ip3-Moose.py
#always indicate which units you are using: cm, uM, sec
#see example in crossComptSimpleReac.py
#Consider functions for (1) reactions (5-6 lines each), (2) pools (2 lines each)
# mglu-param.g stores parameters used in simulation
import os
os.environ['NUMPTHREADS'] = '1'
import numpy as np
import matplotlib.pyplot as plt
plt.ion()

import moose 

import mglu_param as par
import mglu_graphs as graph

#Glu-mGLuR + G <-> G-Glu-mGluR -> aG + Glu-mGluR (uses Vmax2 Km2)
# aG + PLC <-> aPLC-aG (uses G2)
# aPLC + PIP2 <-> aPLC-PIP2 -> aPLC +IP3+ DAG (uses Vmax1 Km1)

#Create a compartment with volume to hold all molecules
cell="/chem"
compt=moose.CubeMesh(cell)
moose.showfield(compt)
compt.volume=par.Volume
#Notice that dx, dy, dz updated automatically
moose.showfield(compt)

# Binding of glutamate to receptor is very fast.  Instead of modeling 
#this explicitly, create a element with transient value of bound mGluR.  
simdt=1e-5
pulsetime=1.0
stimon = [par.K_mGluR for i in range(int(pulsetime/simdt))]
stim=np.append(np.insert(stimon,0,0),0)
stimtable=moose.StimulusTable(compt.path+'/stim')
stimtable.startTime = 1
stimtable.stopTime = 2
stimtable.stepSize=0
stimtable.vector=stim
Glu_mGluR=moose.BufPool(compt.path+'/Glu_mGluR')
#This Glu_mGluR is the bound receptor, which acts as enzyme
moose.connect( stimtable, 'output', Glu_mGluR, 'setConcInit' )

# Create pools of molecules.  One element is required for each molecule.
#Conservepool calculates concentration by subtracting other forms of molecule.
#I don't think there are conserve pools in moose, so create regular pools

Gabg=moose.Pool(cell+'/Gabg')
Gabg.concInit=par.K_G
PLC=moose.Pool(cell+'/PLC')
PLC.concInit=par.K_PLC
PIP2=moose.Pool(cell+'/PIP2')
PIP2.concInit=par.K_PIP2
 
aG=moose.Pool(compt.path+'/aG')   #pool of active GaGTP
aG.concInit=par.aG_init

aPLC_aG=moose.Pool(compt.path+'/aPLC_aG') #pool of active PLC, bound to G_alpha
aPLC_aG.concInit=par.aPLC_aG_init

IP3=moose.Pool(compt.path+'/IP3')  # pool of IP3 (created from PIP2)
IP3.concInit=par.IP3_init

#verify that elements are setup correctly, and
#show different ways of printing information
moose.le(aG)
aG.volume
aPLC_aG.concInit
moose.showfield(IP3)
#Notice nInit is initialized based on volume, which is derived from compt

#Now create bimolecular and enzyme reactions that change molecule conc
# Create reaction, assign rate constant (no volume or conc)
#   then set up messages between substrates, products and reactions.

#Michaelis-Menton formulation: complex is not be explicit
# Glu-mGluR + Gabg <-> GG(lu-)mGluR -> Ga + Glu-mGluR + Gbg 
GGmGluR=moose.MMenz(compt.path+'/GGmGluR')
GGmGluR.kcat=par.Vmax2
GGmGluR.Km=par.Km2
moose.connect(GGmGluR,'sub',Gabg,'reac')
moose.connect(Glu_mGluR,'nOut',GGmGluR,'enzDest')
moose.connect(GGmGluR,'prd', aG,'reac')

# Next reaction is bimolecular - one aG to one PLC 
# aG + PLC <-> aPLC-aG (uses G2)
PLCrxn=moose.Reac(cell+'/PLCrxn')
PLCrxn.Kf=par.G2f
PLCrxn.Kb=par.G2b
#Note that Moose takes care of sending reac fields into pools
moose.connect(PLCrxn,'sub', PLC, 'reac')
moose.connect(PLCrxn,'sub', aG, 'reac')
moose.connect(PLCrxn,'prd', aPLC_aG, 'reac')
moose.showmsg(PLCrxn)

# Second enzyme reaction uses the enzyme object. This explicitly
#calculates the quantity of enzyme substrate complex.  
#
# aPLC-aG + PIP2 <-> aPLC-PIP2 -> aPLC-aG+IP3+ DAG 
PLC_PIP2=moose.Enz(compt.path+'/PLC_PIP2')
plcpip2cplx=moose.Pool(compt.path+'/PLC_PIP2/cplx')
PLC_PIP2.kcat=par.Vmax1
PLC_PIP2.k1=par.kfplc
PLC_PIP2.k2=par.kbplc
moose.connect(PLC_PIP2, 'sub', PIP2, 'reac')
moose.connect(PLC_PIP2, 'prd', IP3, 'reac')
moose.connect(PLC_PIP2, 'enz', aPLC_aG, 'reac')
moose.connect(PLC_PIP2, 'cplx', plcpip2cplx, 'reac')

#verify that messages have been setup correctly
moose.showmsg(compt.path+'/PLC_PIP2')

#Build and configure the solvers
ksolve = moose.Ksolve( '/chem/ksolve' )
stoich = moose.Stoich( '/chem/stoich' )
stoich.compartment = compt
stoich.ksolve=ksolve
stoich.path=cell+"/##"
moose.setClock( 16, simdt )

#Adjust time step; use larger output time step to improve simulation speed
# simulate at dt=1e-5 sec, but output to graphs at slower rate: 1e-3
plotdt=1e-3
runtime=10.0

#create tables to hold data for graphing (or writing to file)
moltables=graph.tables(cell,plotdt)
# MOOSE autoschedules everything
moose.reinit()
moose.start( runtime )

graph.graphs(moltables,runtime)

